#/*
#//  The Natto View
#//
#//  Copyright (c) 1995-1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
#//                           Keio University (Matsushita Lab.)
#//
#//  Please first read the full copyright statement in the file LICENSE.
#//  The code in this file may be used, copied and redistributed under the
#//  same terms as perl itself (see the files README, Copying and Artistic
#//  in the perl distribution).
#*/

# Response headers
# CT: 'Content-Type:'
# LM: 'Last-Modified:'
# GA: 'Date:' of the last GET request
# HA: 'Date:' of the last HEAD request
# EX: 'Expires:'
# RE: 'Location:' (Redirection)

# Page content information
# BA: <base href="...">
# TI: <title>...</title>
# NL: number of links in the page
# Lx: <a href="...">
# Ax: <a>...</a>

package LinkDB;
use subs qw(lock);

use strict;
use GDBM_File;
use FileHandle;
use Fcntl;

sub LOCK_SH  { 1 }
sub LOCK_EX  { 2 }
sub LOCK_UN  { 8 }


sub new
{
    my($class, $file) = @_;
    my $self = {};
    bless $self, $class;
    $self->{'file'} = $file;
    $self;
}


sub DESTROY
{
    my $self = shift;
    untie(%$self->{'hash'});
}


sub set
{
    my($self, $key, $record) = @_;
    my $file = $self->{'file'};
    my %hash;
#    my $fh = new FileHandle;
#    $fh->open("$file.lock", "w+");
#    lock($fh, LOCK_EX, sub {
        tie(%hash, 'GDBM_File', "$file", GDBM_WRCREAT, 0644);
        $hash{$key} = join("\0", %$record);
	untie(%hash);
#    });
}


sub get
{
    my($self, $key) = @_;
    my $file = $self->{'file'};
    my %hash;
    my %record;
#    my $fh = new FileHandle;
#    $fh->open("$file.lock", "w+");
#    lock($fh, LOCK_SH, sub {
        tie(%hash, 'GDBM_File', "$file", GDBM_WRCREAT, 0644);
        %record = split(/\0/, $hash{$key});
	untie(%hash);
#    });
    \%record;
}


sub lock
{
    my($file, $lockflg, $proc) = @_;
    flock($file, $lockflg);
    &$proc;
    flock($file, LOCK_UN);
}

1;
