#/*
#//  The Natto View
#//
#//  Copyright (c) 1995-1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
#//                           Keio University (Matsushita Lab.)
#//
#//  Please first read the full copyright statement in the file LICENSE.
#//  The code in this file may be used, copied and redistributed under the
#//  same terms as perl itself (see the files README, Copying and Artistic
#//  in the perl distribution).
#*/

package LinkParser;
@ISA = qw(HTML::Parser);

use strict;


sub new
{
    my($class, $record) = @_;
    my $self = {};
    bless $self, $class;
    $self->{'record'} = $record;
    ${$self->{'record'}}{'NL'} = 0;
    $self;
}


sub declaration
{
    my($self, $decl) = @_;
}


sub start
{
    my($self, $tag, $attr) = @_;
    
    if ($tag eq 'title') {
	$self->{'hx'} = $tag;
    }
    elsif ($tag eq 'a') {
	$self->{'a_href'}  = $attr->{'href'};
	$self->{'a_title'} = $attr->{'title'};
	$self->{'a_rel'}   = $attr->{'rel'};
    }
    elsif ($tag eq 'frame') {
	$self->hyperlink($attr->{'src'}, $attr->{'src'});
    }
    elsif ($tag eq 'img') {
	my $src = $attr->{'src'};
	my $alt = $attr->{'alt'};
	if ($alt) {
#	    $self->hyperlink($src, $alt);
	    $self->text($alt);
	}
       else {
#	    $self->hyperlink($src, "[$src]");
	    $self->text("[$src]");
	}
    }
    elsif ($tag eq 'base') {
	${$self->{'record'}}{'BA'} = $attr->{'src'};
    }
}


sub end
{
    my($self, $tag) = @_;
    
    if ($tag eq 'title') {
	${$self->{'record'}}{'TI'} = $self->{'heading'};
	undef $self->{'hx'};
    }
    elsif ($tag eq 'a') {
	my $href = $self->{'a_href'};
	$self->hyperlink($href, $self->{'anchor'},
	                 $self->{'a_title'}, $self->{'a_rel'}) if $href;
	undef $self->{'a_href'};
	undef $self->{'a_title'};
	undef $self->{'a_rel'};
	undef $self->{'anchor'};
    }
}


sub text
{
    my $self = shift;
    if ($self->{'hx'}) {
	$self->{'heading'} .= $_[0];
    }
    elsif ($self->{'a_href'}) {
	$self->{'anchor'} .= $_[0];
    }
}


sub hyperlink
{
    my($self, $link, $anchor) = @_;
    $link =~ /^mailto:/ && return;
    my $n = ++${$self->{'record'}}{'NL'};
    $link =~ s/\s+/ /g;
    $link =~ s/^\s+//; 
    $link =~ s/\s+$//; 
    $anchor =~ s/\s+/ /g;
    $anchor =~ s/^\s+//; 
    $anchor =~ s/\s+$//; 
    ${$self->{'record'}}{"L$n"} = $link;
    ${$self->{'record'}}{"A$n"} = $anchor;
}

1;
