/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifdef __GNUG__
#pragma implementation
#endif

#include "linkview.h"
#include "NodeMapperLU.h"

#include <math.h>
#include <ctype.h>
#include <strstream>
#include "Web.h"

#define HOURS  (60 * 60)
#define DAYS   (24 * HOURS)
#define YEARS  (365 * DAYS)


NodeMapperLU::NodeMapperLU():
    NodeMapper("Last Update (days ago)", "URL (1st letter)"),
    upto_(time(NULL)), since_(upto_ - 1 * YEARS)
{}


void
NodeMapperLU::layout(
    Web::node* vp)
{
    if (!vp) return;
    NodeInfo& info = vp->info();
    string    file = vp->key().last_word();
    
    info.eye = info.eyebrow = info.mouth = 0.5;
    info.x = info.y = 0;

    if (info.last_modified > 0) {
	info.x = (float)(info.last_modified - since_) / (upto_ - since_);
	if      (info.x < 0) info.x = 0.0;
	else if (info.x > 1) info.x = 1.0;
	info.x = pow(info.x, 10);
    }
    else if (vp->in().size()) {
	for (Web::link_iterator ei = vp->in().begin();
	     ei != vp->in().end(); ei++)
	    info.x += (*ei)->from()->info().x;
	info.x /= vp->in().size();
    }
    else {
	info.x = 0.5;
    }
    for (unsigned i = 0, j = 0; i < file.length(); i++) {
	int ch = toupper(file[i]);
	if (isalpha(ch) || isdigit(ch)) {
	    if (isdigit(ch)) ch = 'Z' + 1;
	    info.y += (ch - 'A') * pow(26, - (double) j++);
	}
    }
    info.y /= 27;
    info.z  = z_step() * info.doi + log(vp->out().size() + 1) * 0.02;
    float r = def_radius() * log(vp->in().size() + vp->out().size() + 2);
    if (info.r != r) {
	info.r = r;
	info.icon_changed = true;
    }
}


bool
NodeMapperLU::x_label(
    int     nth,
    float&  pos,
    string& label)
{
    time_t grid[] = {
	365 * DAYS,
	 80 * DAYS,
	 60 * DAYS,
	 50 * DAYS,
	 40 * DAYS,
	 30 * DAYS,
	 20 * DAYS,
	 15 * DAYS,
	 10 * DAYS,
	  7 * DAYS,
	  5 * DAYS,
	  2 * DAYS,
	  0 * DAYS,
    };
    if (nth < 0 || sizeof(grid)/sizeof(grid[0]) < (unsigned) nth) {
	pos = 0.0;
	label = "";
	return false;
    }
    time_t t = upto_ - grid[nth];
    pos = pow((double)(t - since_) / (upto_ - since_), 10);
    strstream buf;
    buf << (upto_ - t) / DAYS << ends;
    label = buf.str();
    buf.freeze(false);
    return true;
}


bool
NodeMapperLU::y_label(
    int     nth,
    float&  pos,
    string& label)
{
    if (nth < 0 || 13 < nth) {
	pos = 0.0;
	label = "";
	return false;
    }
    nth *= 2;
    pos = (float) nth / 27;
    if (nth == 26) label = "0-9";
    else           label = 'A' + nth;
    return true;
}


