/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifdef __GNUG__
#pragma implementation
#define __SINST__  /* Why need ? */
#endif

#include "linkview.h"
#include "WebExtenderHT.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <strstream>
#include <WWWApp.h>
#include <WWWInit.h>
#include "NodeMapper.h"


bool    LinkParser::class_initialized = false;
HTList* LinkParser::encoders   = NULL;
HTList* LinkParser::converters = NULL; 


static HTErrorMessage emes[] = { HTERR_ENGLISH_INITIALIZER };

void
LinkParser::class_init()
{
    class_initialized = true;

    HTProxy_getEnvVar();
    HTAccessInit();
    HTTransportInit();
    HTNet_setMaxSocket(6);
    HTDNS_setTimeout(60);

    HTFileInit();
    HTIconInit(NULL);
    HTMIMEInit();
    // HTHeader_addParser("*", NO, header_handler);

    converters = HTList_new();
    HTConverterInit(converters);
    HTFormat_setConversion(converters);

    encoders = HTList_new();
    HTEncoderInit(encoders);
    HTFormat_setTransferCoding(encoders);
#if 0
    HTAuthCall_add("basic",  HTBasic_parse, HTBasic_generate, HTBasic_delete);
    HTAuthCall_add("digest", HTDigest_parse,HTDigest_generate,HTDigest_delete);
    anchor = HTHomeAnchor();
    if (logfile) HTLog_open(logfile, YES, YES);

    if (HTAlert_interactive()) {
	history = HTHistory_new();
	HTMemoryCache_register(HTMemoryCache);
    }
#endif
#if 0    
    HTNet_addAfter(ht_authenticate, NULL, NULL, HT_NO_ACCESS, HT_FILTER_LAST);
    HTNet_addAfter(ht_redirect,  NULL, NULL, HT_PERM_REDIRECT, HT_FILTER_LAST);
    HTNet_addAfter(ht_redirect,  NULL, NULL, HT_TEMP_REDIRECT, HT_FILTER_LAST);
#endif    
    HTNet_addAfter(ht_terminate, NULL, NULL, HT_ALL, HT_FILTER_LAST);
}


LinkParser::LinkParser(
    WebExtenderHT* extenderht,
    Web::node*     vp):
    extenderht_(extenderht), vp_(vp), HTMLParser(HTRequest_new())
{
    if (!class_initialized) class_init();
    if (!vp_) return;
    HTRequest_setContext(htrequest(), this);
#if 0    
    struct timeval tv = { 10, 0 };
    HTEventrg_registerTimeout(&tv, htrequest(), ht_timeout, NO);
#endif
    HTRequest_setPreemptive(htrequest(), NO);
    HTRequest_setOutputFormat(htrequest(), WWW_HTML);
    HTRequest_setOutputStream(htrequest(), htstream());
}


LinkParser::~LinkParser()
{
    HTRequest_delete(htrequest());
    extenderht_->remove_parser(this);
}


void
LinkParser::load()
{
    vp_->info().n_images = 0;
    HTRequest_setReloadMode(htrequest(), HT_CACHE_FLUSH);
    HTLoadAbsolute(vp_->key().absolute().c_str(), htrequest());
}


void
LinkParser::run()
{
    strstream msg;
    msg << "Loading " << vp_->key() << "..." << ends; 
    message(msg.str());
    msg.freeze(false);
    load();
}


void
LinkParser::message(
    string str)
{
    extenderht_->web()->message(str);
}


int
LinkParser::authenticate(
    int status)
{
    vp_->info().http_status = status;
#if 0
    HTRequest* req = htrequest();
    if (HTAuth_parse(req)) {
	if (HTLog_isOpen()) HTLog_add(req, status);
	load();
    }
    else {
#endif
	message("Access denied.");
#if 0
    }
#endif
    return HT_ERROR;
}


int
LinkParser::redirect(
    int status)
{
    vp_->info().http_status = status;
#if 0
    HTRequest* req = htrequest();
    if (HTLog_isOpen()) HTLog_add(req, status);
    if (HTRequest_retrys(req)) {
	HTRequest_doRetry(req);
	HTParentAnchor* htanchor = HTRequest_anchor(req);
	char* location = HTAnchor_location(htanchor);
	if (location && location[0])
	    vp_ = extenderht_->web()->redirect(vp_, URL(location));
	// load();
    }
    else {
	message("Too many redirections detected.");
    }
#endif
    return HT_OK;
}


int
LinkParser::terminate(
    int status)
{
    HTRequest* req = htrequest();
    NodeInfo& info = vp_->info();
    info.http_status = status;

    if (status == HT_LOADED) {
	HTParentAnchor* htanchor = HTRequest_anchor(req);
	info.content_type   = HTAnchor_format(htanchor)->name;
	info.content_length = HTAnchor_length(htanchor);
	info.last_get       = HTAnchor_date(htanchor);
	info.last_head      = info.last_get;
	info.expires        = HTAnchor_expires(htanchor);
	info.last_modified  = HTAnchor_lastModified(htanchor);
	info.scanned        = true;
	if (info.content_type.substr(0, 4) == "text")
	    info.content = content();
	info.fill_variables();
	strstream msg;
	msg << "Read success! " << vp_->key() << ends;
	message(msg.str());
	msg.freeze(false);
    }
    else if (status == HT_TEMP_REDIRECT || status == HT_PERM_REDIRECT) {
	// info.disabled = true;
	HTResponse* response = HTRequest_response(req);
	HTAnchor* htanchor =
	    response ? HTResponse_redirection(response): (HTAnchor*) 0;
	char* location = htanchor ? HTAnchor_address(htanchor): (char*) 0;
	if (location && location[0]) {
	    vp_ = extenderht_->web()->redirect(vp_, URL(location));
	    load();
	}
	strstream msg;
	msg << "Redirecting to " << location << "..." << ends;
	message(msg.str());
	msg.freeze(false);
    }
    else if (status == HT_ERROR) {
	HTList* list = HTRequest_error(req);
	if (HTList_isEmpty(list)) {
	    strstream msg;
	    msg << "Retrying " << vp_->key() << "..." << ends; 
	    message(msg.str());
	    msg.freeze(false);
	    load();
	} else {
	    info.disabled = true;
	    HTError* err = (HTError*) HTList_firstObject(list);
	    int index = HTError_index(err);
	    strstream msg;
	    msg << "ERROR: " << emes[index].code << ' ' << emes[index].msg
		<< ". " << vp_->key() << ends;
	    message(msg.str());
	    msg.freeze(false);
	}
    }
    extenderht_->web()->mapper()->layout(vp_);
    for (Web::link_iterator ei = vp_->out().begin();
	 ei != vp_->out().end(); ei++)
	extenderht_->web()->mapper()->layout((*ei)->to());
    extenderht_->web()->update(WebNotify::sync);
    return HT_OK;
}


int
LinkParser::timeout()
{
    vp_->info().http_status = -1;
    message("Request timed out.");
    return 0;
}


void
LinkParser::title(
    string str)
{
    vp_->info().title = str;
    vp_->info().label_changed = true;
}


void
LinkParser::hyperlink(
    URL    url,
    string anchor,
    string title,
    string ltype)
{
    URL u = url.parse(URL::all_fields & ~URL::frag);
    string frag = url.parse(URL::frag);
    Web* web = extenderht_->web();
    Web::node* vp_to = NULL;
    for (;;) {
	Web::node v(u, NodeInfo(title.length() ? title: u.last_word()));
	Web::node_iterator vi = web->nodes().find(&v);
	vp_to = (vi != web->nodes().end()) ? *vi: web->insert(v);
	if (!vp_to->info().is_redirect()) break;
	u = vp_to->info().redirect;
    }
    if (vp_ != vp_to) {
	LinkInfo info(frag, anchor, ltype);
	Web::link_iterator ei;
	for (ei = vp_->out().begin();
	     ei != vp_->out().end() && (*ei)->info() != info; ei++)
	    ;
	if (ei == vp_->out().end()) {
	    web->insert(Web::link(vp_, vp_to, info));
	    if (vp_to->info().doi < vp_->info().doi - 1)
		vp_to->info().doi = vp_->info().doi - 1;
	    web->mapper()->layout(vp_);
	    web->mapper()->layout(vp_to);
	}
    }
    // web->update(WebNotify::ignorable);
}


void
LinkParser::image(URL, string)
{
    ++vp_->info().n_images;
}



WebExtenderHT::WebExtenderHT(
    Web*       web,
    Web::node* first,
    int        depth):
    WebExtender(web, first, depth)
{}


WebExtenderHT::WebExtenderHT(
    Web* web,
    URL  first,
    int  depth):
    WebExtender(web, first, depth)
{}


void
WebExtenderHT::run()
{
    invoke_parser(new LinkParser(this, first_node()));
}


void
WebExtenderHT::invoke_parser(
    LinkParser* parser)
{
    parsers_.insert(parser);
    parser->run();
}


void
WebExtenderHT::remove_parser(
    LinkParser* parser)
{
    parsers_.erase(parser);
    // delete parser;
    if (parsers_.empty()) delete this;
}
