/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifndef WEBEXTENDERHT_H
#define WEBEXTENDERHT_H
#ifdef __GNUG__
#pragma interface
#endif

#include <set>
#include "www.h"
#include "WebIO.h"

class WebExtenderHT;


class LinkParser: public HTMLParser
{
 public:
    static void class_init();
    
    LinkParser(WebExtenderHT*, Web::node*);
    virtual ~LinkParser();

    void run();
    void message(string);

 protected:
    void load();

    void title(string);
    void hyperlink(URL url, string anchor, string title, string type);
    void image(URL url, string alt);
#if 0
    void heading(h_level, string);
    void movie(URL url, string alt);
    void button(URL url, string method);
    void frame(URL url);
    void object(URL url);
#endif    
    int timeout();
    int authenticate(int status);
    int redirect(int status);
    int terminate(int status);
    
    static int ht_timeout(HTRequest* r)
	{ return ((LinkParser*) HTRequest_context(r))->timeout(); }
    static int ht_authenticate(HTRequest* r, HTResponse*, void*, int s)
	{ return ((LinkParser*) HTRequest_context(r))->authenticate(s); }
    static int ht_redirect(HTRequest* r, HTResponse*, void*, int s)
	{ return ((LinkParser*) HTRequest_context(r))->redirect(s); }
    static int ht_terminate(HTRequest* r, HTResponse*, void*, int s)
	{ return ((LinkParser*) HTRequest_context(r))->terminate(s); }
    
private:
    WebExtenderHT* extenderht_;
    Web::node*     vp_;
	
    static bool    class_initialized;
    static HTList* converters;
    static HTList* encoders;
};

    
class WebExtenderHT: public WebExtender
{
  public:
    WebExtenderHT(Web* web, Web::node* start, int depth);
    WebExtenderHT(Web* web, URL start, int depth);

    virtual void run();
    void invoke_parser(LinkParser*);
    void remove_parser(LinkParser*);

 private:
#ifdef STRICT_STL
    set<LinkParser*, less<LinkParser*> > parsers_;
#else
    set<void*, less<void*> > parsers_;
#endif
};

#endif /* WEBEXTENDERHT_H */
