/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                      Keio University (Matsushita Lab.)
//                      All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifndef WEBFRAME_H
#define WEBFRAME_H
#ifdef __GNUG__
#pragma interface
#endif

#include <time.h>
#include "Web.h"
#include "ptrvector.h"

class WebView;
class WebReader;
class WebWriter;
class WebExtender;


class WebFrame: public observer<WebNotify>
{
 public:
    WebFrame(Web* web, string server, u_short port);
    virtual ~WebFrame() {}

    // operations and manipulations (called by WebView)
    void focus(URL url);
    void focus(Web::node*);
    void lift_up()       { web()->lift_up(); }
    void lift_down()     { web()->lift_down(); }
    void lift_down_all() { web()->lift_down_all(); }
    void reload();
    virtual void browse(Web::node* = NULL) {}
    void scan(Web::node* = NULL);

    void     active_view(WebView* view) { active_view_ = view; }
    WebView* active_view()              { return active_view_; }
    WebView* rootview()                 { return rootview_; }

    virtual void update(WebNotify);
    virtual void update(observable<WebNotify>*, WebNotify n) { update(n); }

    // invoke I/O objects
    virtual WebExtender* invoke_extender(Web::node*, int) { return 0; }
    virtual WebReader*   invoke_reader(istream*);
    virtual WebWriter*   invoke_writer(ostream*);

    virtual void message(string s) { cerr << s << endl; }
    virtual void show_node_info();

    Web*    web()          { return web_; }
    string  server() const { return server_; }
    u_short port()   const { return port_; }
    
 protected:
    void rootview(WebView* view) { rootview_ = active_view_ = view; }

 private:
    Web*     web_;
    string   server_;
    u_short  port_;
    WebView* rootview_;
    WebView* active_view_;
    time_t   last_update_;
};

#endif /* WEBFRAME_H */
