/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifndef WEBFRAMEX_H
#define WEBFRAMEX_H
#ifdef __GNUG__
#pragma interface
#endif

#include <Xm/XmAll.h>
#include "WebFrame.h"
#include "linkview.h"
#include "xtinter.h"

class WebReaderX;
class WebWriterX;
class WebExtenderX;


//////////////////////////////////////////////////////////////////////////////
//  WebFrameX -- 
//
class WebFrameX: public WebFrame
{
 public:
    // primitive functions
    WebFrameX(Web* web, string server, u_short port, Widget toplevel);
    virtual ~WebFrameX() {}

    // window management
    virtual void update(WebNotify);
    virtual void message(string);
    
    // File menu
    void save();
    void load();
    void quit() { Quit(); }

    // Options menu
    void access_via_linkdb();
    void access_direct_http();
    void mapper_uu();
    void mapper_lu();
#ifdef PABLO
    void mapper_gen();
#endif

    // Window menu
    void anchorbox();
    void tourbox();
    void config_mapper();

    // Help menu
    void about();

    // WWW browser control
    enum browser_t { mozilla, mosaic };
    browser_t    browser() const      { return browser_; }
    void         browser(browser_t b) { browser_ = b; }
    virtual void browse(Web::node* = NULL);

    enum access_t { via_linkdb, direct_http };
    access_t access_mode() const     { return access_mode_; }
    void     access_mode(access_t m) { access_mode_ = m; }

    virtual WebExtender* invoke_extender(Web::node*, int);
    virtual WebReader*   invoke_reader(istream*);

    // member access
    XtAppContext app_context() { return XtWidgetToApplicationContext(shell_); }
    Widget       toplevel()    { return shell_; }

 protected:
    virtual void show_node_info();
    void         anchorbox_create();

    // subroutines of browse()
    void mozilla_open(URL);
    void mosaic_get(URL);

    // Motif callback interface
#define XTINTER_CLASS WebFrameX    
    XTINTER_FAST(save_ok,             XmFileSelectionBoxCallbackStruct*)
    XTINTER_FAST(save_cancel,         XmFileSelectionBoxCallbackStruct*)
    XTINTER_FAST(load_ok,             XmFileSelectionBoxCallbackStruct*)
    XTINTER_FAST(load_cancel,         XmFileSelectionBoxCallbackStruct*)
#if 0
    XTINTER_FAST(file_search,         XmFileSelectionBoxCallbackStruct*)
#endif
    XTINTER_FAST(sitebox_activate,    XtPointer)
    XTINTER_FAST(sitebtn_activate,    XtPointer)
    XTINTER_FAST(anchorbox_destroy,   XtPointer)
    XTINTER_FAST(anchorbox_action,    XmListCallbackStruct*)
    XTINTER_FAST(anchorbox_selection, XmListCallbackStruct*)
    XTINTER_FAST(anchorbox_focus,     XtPointer)
    XTINTER_FAST(anchorbox_browse,    XtPointer)
    XTINTER_FAST(anchorbox_scan,      XtPointer)
    XTINTER_FAST(anchorbox_tour,      XtPointer)
    XTINTER_FAST(anchorbox_close,     XtPointer)
    XTINTER_FAST(anchorbox_sitebox_activate, XtPointer)
    XTINTER_FAST(anchorbox_sitebtn_activate, XtPointer)
#undef XTINTER_CLASS
    
#define MENU_MACRO_CLASS WebFrameX    
#include "menubar.menu"

 private:
    Widget    shell_;    // see Nattoview.ad
    Widget    form_;
    Widget    menubar_;
    Widget    sitebar_;
    Widget    sitelabel_;
    Widget    sitebox_;
    Widget    sitebtn_;
    Widget    glframe_;
    Widget    statbar_;
    Widget    statbox_;
    Widget    filebox_;
    Widget    aboutbox_;
    browser_t browser_;  // WWW browser
    access_t  access_mode_;

    Widget    anchorbox_popup_;
    Widget    anchorbox_form_;
    Widget    anchorbox_sitebar_;
    Widget    anchorbox_sitelabel_;
    Widget    anchorbox_sitebox_;
    Widget    anchorbox_sitebtn_;
    Widget    anchorbox_list_;
    Widget    anchorbox_btnbar_;
    Web::link_vector anchorbox_items_;
    Web::node* anchorbox_prev_active_;
};

#endif /* WEBFRAMEX_H */
