/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifdef __GNUG__
#pragma implementation
#endif

#include "linkview.h"
#include "WebIOX.h"

#include <fstream>
#include <strstream>
#include "copy_line.h"
#include "Web.h"

#define CRLF "\015\012"


WebReaderX::WebReaderX(
    Web*         web,
    XtAppContext app,
    ifstream*    in,
    int          flags):
    WebReader(web, (istream*) in, flags), app_(app), id_(0)
{}


void
WebReaderX::run()
{
    ifstream* ifs = DCAST(ifstream*, in());
    if (!ifs) {
	WebReader::run();
	return;
    }
    id_ = XtAppAddInput(app_context(), ifs->rdbuf()->fd(),
			(XtPointer) XtInputReadMask,
			(XtInputCallbackProc) xt_read_callback,
			(XtPointer) this);
}


void
WebReaderX::read_callback()
{
    if (read_line()) {
	// OK, continue!
	return;
    }
    else {
	// EOF or some trouble
	XtRemoveInput(id_);
	delete this;
    }
}


WebExtenderX::WebExtenderX(
    Web*         web,
    XtAppContext app,
    Web::node*   first,
    int          depth,
    string       server,
    u_short      port,
    int          flags):
    WebExtender(web, first, depth, flags), app_(app),
    server_(server), port_(port),
    readerx_(NULL), in_(NULL), id_(0)
{}


WebExtenderX::WebExtenderX(
    Web*         web,
    XtAppContext app,
    URL          url,
    int          depth,
    string       server,
    u_short      port,
    int          flags):
    WebExtender(web, url, depth, flags), app_(app),
    server_(server), port_(port),
    readerx_(NULL), in_(NULL), id_(0)
{}


WebExtenderX::~WebExtenderX()
{
    if (id_) XtRemoveInput(id_);
    if (readerx_) {
	// Don't delete readerx_ here!  It is already deleted by itself.
	readerx_ = NULL;
	in_->seekp(0, ios::end);
	*in_ << "QUIT" << CRLF << flush;
	in_->seekg(0, ios::end);
	strstream buf;
	copy_line(*in_, buf); buf << ends;
    }
    if (in_) {
	in_->close();
	delete in_;
	in_ = NULL;
    }
}


void
WebExtenderX::run()
{
    web()->message("Connecting LinkDB...");
    if (!connect(server_, port_)) {
	delete this;
	return;
    }
    web()->message("Sending request...");
    in_->seekp(0, ios::end);
    *in_ << "SCAN " << first_node()->key() << ' ' << depth() << CRLF << flush;
    in_->seekg(0, ios::end);
    id_ = XtAppAddInput(app_context(), in_->rdbuf()->fd(),
			(XtPointer) XtInputReadMask,
			(XtInputCallbackProc) xt_read_status,
			(XtPointer) this);
}


// connecting to the LinkDB
bool
WebExtenderX::connect(
    string  thathost,  // host name of LinkDB
    u_short port)      // port number
{
    in_ = new socketstream(thathost.c_str(), port);
    strstream header;
    if (!copy_line(*in_, header)) {
	web()->message("ERROR: Can't read status field.");
	return false;
    }
    header << ends;
    DEBUGOUT(header.str());
    header.freeze(false);
    
    int status;
    header >> status;
    if (status / 100 != 2) {
	web()->message("ERROR: Status field not 2XX.");
	return false;
    }
    return true;
}


void
WebExtenderX::read_status()
{
    XtRemoveInput(id_);
    id_= 0;

    strstream buf;
    copy_line(*in_, buf); buf << ends;

    int status = 0;
    buf >> status;
    if (status / 100 != 2) {
	strstream msg;
	msg << "ERROR: Response status " << status << '.' << ends;
	web()->message(msg.str());
	msg.freeze(false);
	delete this;
	return;
    }

    web()->message("Reading from LinkDB...");
    readerx_ = new Reader(this, web(), app_context(), in_);
    readerx_->run();
    // Don't worry, readerx_ will call extenderx_'s delete.
}
