/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                      Keio University (Matsushita Lab.)
//                      All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifndef KANJI_H
#define KANJI_H

#include <strstream>
#include <string>
#include <stddef.h>


enum { KANJI_ASCII = 0, KANJI_JIS = 1,
       KANJI_SJIS  = 2, KANJI_EUC = 4, KANJI_ALL = 7 };

int guess_kanji_code(char* str, size_t len, int mask);


class converterbuf : public streambuf
{
 public:
    converterbuf(ostream* out): out_(out) {}
    int sync();
    int overflow(int ch);
//    int fd() { return out_ ? out_->rdbuf()->fd: -1; }

 protected:
    virtual int putch(unsigned char ch) { *out_ << ch; return ch; }
    ostream* out_;
};


class s2ebuf: public converterbuf
{
 public:
    s2ebuf(ostream* out): converterbuf(out), state_(first), prev_('\0') {}

 private:
    int  putch(unsigned char);
    void convert(unsigned char&, unsigned char&);
    enum { first, second } state_;
    unsigned char prev_;
};


class j2ebuf: public converterbuf
{
 public:
    j2ebuf(ostream* out): converterbuf(out), state_(ascii), prev_('\0') {}

 private:
    int putch(unsigned char);
    enum {
	ascii, in_esc, in_esc_dollar, first, second, out_esc, out_esc_paren
    } state_;
    unsigned char prev_;
};


class s2estream: public ostream
{
 public:
    s2estream(ostream* out): ostream(new s2ebuf(out)) {}
    ~s2estream() { delete rdbuf(); }
};


class j2estream: public ostream
{
 public:
    j2estream(ostream* out): ostream(new j2ebuf(out)) {}
    ~j2estream() { delete rdbuf(); }
};


istream& anytoeuc(istream&, ostream&);
string   anytoeuc(string);


#endif /* KANJI */
