/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                      Keio University (Matsushita Lab.)
//                      All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifndef OBSERVER_H
#define OBSERVER_H
#ifdef __GNUG__
#pragma interface
#endif

//////////////////////////////////////////////////////////////////////////////
//  Model/View (Observer) tool
//

#include "ptrvector.h"

template <class T> class observable;


//////////////////////////////////////////////////////////////////////////////
//  observer -- 
//
template <class T>
class observer
{
 public:
    virtual ~observer() {}

    virtual void update(observable<T>*, T) = 0;
};


//////////////////////////////////////////////////////////////////////////////
//  observable -- 
//
template <class T>
class observable
{
 public:
    typedef ptrvector<observer<T>*> observer_vector;

    observable(): changed_(false) {}
    // virtual ~observable() {}
    
    void attach(observer<T>* v)
    {
	if (find(observers_.begin(), observers_.end(), v) == observers_.end()) 
	    observers_.push_back(v);
    }
    
    void detach(observer<T>* v)
    {
	observer_vector::iterator i =
	    find(observers_.begin(), observers_.end(), v);
	if (i != observers_.end()) observers_.erase(i);
    }
    
    void notify(T info)
    {
	if (!changed_) return;
	for (observer_vector::reverse_iterator i = observers_.rbegin();
	     i != observers_.rend(); i++)
	    (*i)->update(this, info);
	changed_ = false;
    }
  
    bool changed() { return changed_; }
    
 protected:
    void changed(bool f) { changed_ = f; }
    
 private:
    bool            changed_;
    observer_vector observers_;
};

#endif /* OBSERVER */
