/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifndef PTRVECTOR_H
#define PTRVECTOR_H
#ifdef __GNUG__
#pragma interface
#endif

#include <set>
#include <vector>
#include <iterator>
#include "settings.h"

#ifdef STRICT_STL

#define ptrset    set
#define ptrvector vector

#else

template <class BidirectionalIterator, class T>
class cast_iterator: public bidirectional_iterator<T, ptrdiff_t>
{
    typedef cast_iterator<BidirectionalIterator, T> self_type;
    friend bool operator==(self_type x, self_type y)
	{ return x.iter == y.iter; }
    friend bool operator==(BidirectionalIterator x, self_type y)
	{ return x == y.iter; }
    friend bool operator==(self_type x, BidirectionalIterator y)
	{ return x.iter == y; }
    
 protected:
    BidirectionalIterator iter;

 public:
    cast_iterator() {}
    cast_iterator(BidirectionalIterator i): iter(i) {}
    T& operator*() { return (T&) *iter; }
    const T& operator*() const { return (const T&) *iter; }
    operator BidirectionalIterator() const { return iter; }
    
    self_type& operator++()    { ++iter; return *this; }
    self_type  operator++(int) { self_type i = *this; ++iter; return i; }
    self_type& operator--()    { --iter; return *this; }
    self_type  operator--(int) { self_type i = *this; --iter; return i; }
};


template <class T, class Less>
struct ptrset: set<void*, Less>
{
    typedef cast_iterator<set<void*, Less>::iterator, T> iterator;
    iterator find(const T& v)
	{ return (iterator) set<void*, Less>::find(v); }
    // const
};


template <class T>
struct ptrvector: vector<void*>
{
    typedef vector<void*> parent;
    typedef cast_iterator<parent::iterator, T> iterator;
    typedef cast_iterator<parent::const_iterator, T> const_iterator;
    typedef cast_iterator<parent::reverse_iterator, T> reverse_iterator;
    typedef cast_iterator<parent::const_reverse_iterator, T>
	const_reverse_iterator;
//    iterator begin() { return parent::begin(); }
//    iterator end()   { return parent::end(); }
    T& operator[](size_type n) { return *(T*)(parent::begin() + n); }
    const T& operator[](size_type n) const{ return *(T*)(parent::begin() + n);}
    T& back() { return *(T*)(end() - 1); }
    const T& back() const { return *(T*)(end() - 1); }
};

#endif /* STRICT_STL */

#endif /* PTRVECTOR_H */
