/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Keio University (Matsushita Lab.)
//                           All rights reserved.
//  Please first read the full copyright statement in the file LICENSE.
//  Permission to copy this software, to redistribute it on a nonprofit
//  basis, and to use it for any purpose, is hereby granted, provided that 
//  this license information and copyright notice appear in all copies. 
*/

#ifdef __GNUG__
#pragma implementation
#endif

#include "linkview.h"
#include "socketstream.h"

#include <fstream>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>

#ifndef STD
#define STD /* */
#endif
#ifndef DEBUGOUT
#define DEBUGOUT(x)  /* */
#endif


#if !defined(HAVE_GETHOSTNAME) && defined(HAVE_UNAME)
int
gethostname(
    char*  name,
    size_t len)
{
    struct utsname buf;
    int ret = uname(&buf);
    strncpy(name, buf.nodename, len - 1);
    name[len] = '\0';
    return ret;
}
#endif


void
socketstream::open(
    const char* thathost,
    u_short     port)
{
    if (is_open()) close();

    struct hostent* hent = NULL;
    char thishost[128] = "";
    gethostname(thishost, sizeof(thishost));
    thishost[sizeof(thishost) - 1] = '\0';

    if (!(hent = gethostbyname(thishost))) exit(-1);
    struct sockaddr_in cliaddr;
    memset(&cliaddr, 0, sizeof(cliaddr));
    cliaddr.sin_family = hent->h_addrtype;
    memcpy(&cliaddr.sin_addr, hent->h_addr, (size_t) hent->h_length);

    if (!(hent = gethostbyname(thathost))) exit(-1);
    struct sockaddr_in servaddr;
    memset(&servaddr, 0, sizeof(servaddr));

    servaddr.sin_family = hent->h_addrtype;
    servaddr.sin_port   = htons(port);
    memcpy(&servaddr.sin_addr, hent->h_addr, (size_t) hent->h_length);

    DEBUGOUT(thishost << " (" << inet_ntoa(cliaddr.sin_addr) << ") -> "
	     << hent->h_name << ':' << port << " ("
	     << inet_ntoa(servaddr.sin_addr) << ')');

    int sock = -1;
    if ((sock = socket(servaddr.sin_family, SOCK_STREAM, 0)) < 0) {
	DEBUGOUT("Can't open socket.");
	return;
    }
    if (bind(sock, (struct sockaddr*) &cliaddr, sizeof(cliaddr)) < 0) {
	STD::close(sock);
	DEBUGOUT("Can't bind.");
	return;
    }
    if (connect(sock, (struct sockaddr*) &servaddr, sizeof(servaddr)) < 0) {
	STD::close(sock);
	DEBUGOUT("Can't connect.");
	return;
    }
    attach(sock);
}
